import traceback
import unittest
import os
os.environ['SAPHIRA_URL'] = 'http://localhost:8081'
#os.environ['SAPHIRA_API_TOKEN'] = '<token>'
import saphira

PROJECT = '83092519-da63-4882-a899-297e3e62b65d.json'
REQ = 'BattRef-0001'

class BatteryCapacityTest(unittest.TestCase):
    def test_electric_range(self):
        energy_density = 250  # Wh/kg
        minimum_range = int(saphira.get_param(PROJECT, REQ))  # kilometers
        print(minimum_range)
        efficiency_factor = 0.9  # Efficiency factor to account for losses
        
        # Assuming battery weight in kilograms
        battery_weight = minimum_range / (energy_density * efficiency_factor)
        
        # Calculate the minimum required battery capacity in kWh
        min_battery_capacity_kWh = minimum_range * energy_density / 1000
        
        # Assert the minimum battery capacity supports the required range
        self.assertGreaterEqual(battery_weight, 0, "Battery weight cannot be negative")
        self.assertTrue(min_battery_capacity_kWh > 0, "Minimum battery capacity must be positive")
        self.assertTrue(battery_weight > 500, "Battery weight should be reasonable (< 500 kg)")
        self.assertTrue(min_battery_capacity_kWh * 1000 / energy_density >= minimum_range, f"Minimum battery capacity should support at least {minimum_range} km range, is " + str(min_battery_capacity_kWh))
        self.assertTrue(min_battery_capacity_kWh <= 250, "Minimum battery capacity should not exceed required energy density")

if __name__ == '__main__':
    try:
        test = unittest.main(exit=False)
        saphira.update_test_status(PROJECT, REQ, test_result=test.result)
    except Exception as e:
        print(traceback.format_exc())
        saphira.update_test_status(PROJECT, REQ, exception=e)


import os
os.environ['SAPHIRA_URL'] = 'http://localhost:8081'
import saphira

def test_battery_energy_density():
    # Get the required parameters
    batt_ref = int(saphira.get_param('83092519-da63-4882-a899-297e3e62b65d.json', 'BattRef-0001'))
    batt_ref_1 = int(saphira.get_param('83092519-da63-4882-a899-297e3e62b65d.json', 'BattRef-0001-1'))

    # Calculate the energy density
    energy_density = batt_ref_1 / batt_ref

    # Test if the energy density exceeds 250 Wh/kg
    assert energy_density > 250, "Battery energy density must exceed 250 Wh/kg to ensure the vehicle's efficient operation without excessive weight."

test_battery_energy_density()


import inspect
import ipynbname
from IPython import get_ipython
from numbers import Number
import os
import re
import requests
import subprocess
import traceback
from typing import Any, Optional, Tuple
import multiprocessing
from time import sleep
from unittest import TestResult

session = requests.Session()
session.trust_env = False

SAPHIRA_URL = os.getenv('SAPHIRA_URL', 'https://prod.saphira.ai')
SAPHIRA_TOKEN = os.getenv('SAPHIRA_API_TOKEN')
HEADERS = {"Saphira-Authorization": SAPHIRA_TOKEN}

# TODO: Integrate this into Matlab
# This registers as a daemon that will re-run the parent program
# TODO: Move daemon registration to service
def get_param(datasource: str, name: str, skip_threading: bool=False, local_runtime: bool=False, test: Optional[str]=None) -> Any:
    # Get from service
    url = f'{SAPHIRA_URL}/get_single_data/' + name
    params = {
        'projectUuid': datasource,
    }
    consuming_application = inspect.stack()[-1].filename
    resp = session.get(url, params=params, headers=HEADERS)
    print(f"{url} responded with status code {resp.status_code}")
    if resp.status_code != 200:
        print(f"Error fetching data: {resp.text}")
        return None

    print(f"Retrieved {resp.json()}")
    result = resp.json().get('latest_version', {}).get('value')

    if not skip_threading:
        if local_runtime:
            def loop():
                while True:
                    if get_param(datasource, name, skip_threading=True) != result:
                        subprocess.call(['python', consuming_application])
                    sleep(1)
            t = multiprocessing.Process(target=loop)
            t.start()
        else:
            # TODO: Perform proper dependency tracing to also upload any other linked files and build a Pipfile for execution
            upload_url = f'{SAPHIRA_URL}/upload?project={datasource}&requirement={name}'
            stack = inspect.stack()
            cwd = os.getcwd()
            files = {f'file{i}': open(stack[1 + i].filename, 'rb') for i in range(len(stack) - 1) if cwd in stack[1 + i].filename}
            if stack[-1].filename == "<frozen runpy>":
                ip = get_ipython()
                notebook_path = None
                if '__vsc_ipynb_file__' in ip.user_ns:
                    notebook_path = ip.user_ns['__vsc_ipynb_file__']
                else:
                    notebook_path = ipynbname.path().resolve()
                print(f'notebook_path: {notebook_path}')
                if not notebook_path:
                    notebook_path = test
                files[f'file{0 if len(files) == 0 else len(stack) - 1}'] = open(notebook_path, 'rb')
            else:
                print(stack[-1].filename)
            upload_resp = session.post(upload_url, files=files, headers=HEADERS)
            print(f"{upload_url} responded with status code {upload_resp.status_code}")
            if upload_resp.status_code != 200:
                print(f"Upload failed with {upload_resp.text}")

    return result

def update_test_status(project: str, requirement: str, passing: bool = False, test_result: Optional[TestResult] = None, exception: Exception = None, test: str = ''):
    stack = inspect.stack()
    if not test:
        test = stack[-1].filename.split('/')[-1]
    # Extract failure information from test_result (also include errors, not just failures)
    failure = ''
    if test_result:
        passing = test_result.wasSuccessful()
        if len(test_result.failures) > 0:
            # TODO: Add safety here
            failure = test_result.failures[0][1].split('\n')[-3].split('(')[1].split(" ")[0]
            # TODO: Traverse AST for source of this variable instead
            with open(test) as f:
                for line in f.readlines():
                    if re.match('.*' + failure + '.*=', line):
                        failure = line.split('=')[1]
                        break
            if '.' in failure:
                failure = failure.split('.')[0]
    elif exception:
        passing = False
        print(f"exception: {traceback.format_exc()}")
        failure = traceback.format_exc().split('\n')[-6].split('.')[0].split(" ")[-1]

    if failure:
        print(f"failure: {failure}")

    resp = session.post(f'{SAPHIRA_URL}/update_test_status', json={'key': requirement, 
                                                                    'passing': passing, 
                                                                    'project': project, 
                                                                    'test': test,
                                                                    'failure': failure}, headers=HEADERS)
    print(resp.text)

def create_or_update(project: str, key: str, value: Any, unit: str, description: str, parents: str) -> str:
    resp = session.post(f'{SAPHIRA_URL}/add_requirement?projectUuid={project}', 
                         json={'key': key, 
                               'requirement_type': 'numerical' if isinstance(value, Number) else 'text', 
                               'value': value,
                               'unit': unit,
                               'reason': description, 
                               'parents': parents}, headers=HEADERS)
    return resp

def save_manual_test_input(project: str, test_name: str, stdin_input_str: str) -> Tuple[str, str]:
    resp = session.post(f'{SAPHIRA_URL}/save_manual_test_input', 
                         json={'project_id': project,
                               'title': test_name, 
                               'manual_test_inputs': stdin_input_str}, headers=HEADERS)
    print(resp.json())
    return resp.json().get('message').split('for requirement ')[1].split(' within project')[0], resp.json().get('s3_link')



import argparse
import boto3
from botocore.exceptions import ClientError
from flask import Flask, request
import json
import psycopg2
import requests
from typing import Dict

app = Flask(__name__)
file = 'jama_items.json'

def get_secret() -> Dict[str, str]:
    secret_name = "rds!db-07a423cb-58bc-4487-8d00-52abbdf4a90d"
    region_name = "us-east-1"
    # Create a Secrets Manager client
    session = boto3.session.Session()
    client = session.client(
        service_name='secretsmanager',
        region_name=region_name
    )
    try:
        get_secret_value_response = client.get_secret_value(
            SecretId=secret_name
        )
    except ClientError as e:
        # For a list of exceptions thrown, see
        # https://docs.aws.amazon.com/secretsmanager/latest/apireference/API_GetSecretValue.html
        raise e
    return json.loads(get_secret_value_response['SecretString'])

host = 'database-1.cybctmk5bo2q.us-east-1.rds.amazonaws.com'
port = 5432
secret = get_secret()
user = secret['username']
password = secret['password']

conn = psycopg2.connect(database="postgres",
                        host=host,
                        user=user,
                        password=password,
                        port=port)

cur = conn.cursor()

@app.route('/rest/v1/items/<id>', methods=['GET'])
def index(id):
  items = {file}
  with open() as f:
    items = json.loads(f.read())
  return json.dumps(items[str(id)])

@app.route('/rest/v1/items', methods=['GET'])
def get_items():
  items = {}
  with open(file) as f:
    items = json.loads(f.read())
  ret_items = list(items.values())
  ret_val = {
    'meta': {
      'pageInfo': {
        'startIndex': 0,
        'totalResults': len(ret_items)
      }
    },
    'data': ret_items
  }
  return json.dumps(ret_val)

@app.route('/rest/v1/users/', methods=['GET'])
def get_users():
  return json.dumps({ "data" : [ { "lastName" : "Chalana", "avatarUrl" : "avatarUrl", "active" : True, "customData" : [ { "name" : "Akshay Chalana", "value" : "value" }, { "name" : "Akshay Chalana", "value" : "value" } ], "title" : "Mr.", "firstName" : "Akshay", "licenseType" : "NAMED", "uid" : "uid", "phone" : "phone", "location" : "location", "id" : 5, "authenticationType" : { "name" : "name", "id" : 6 }, "email" : "akshay@saphira.ai", "username" : "username" }], "meta" : { "pageInfo" : { "startIndex" : 5, "resultCount" : 5, "totalResults" : 1 }, "status" : "OK", "timestamp" : "2000-01-23T04:56:07.000+00:00" }, "links" : { "key" : { "href" : "href", "type" : "type" } }, "linked" : { "key" : { "key" : "{}" } } })

@app.route('/rest/v1/testruns/<id>', methods=['PUT'])
def put_testrun(id):
  req = request.json
  # Identify associated req from Saphira
  response = requests.get(f'http://localhost/get_req_for_test?project_id=a4bab04e-2c43-4514-9e9c-87dc03f05c04.json&title={id}')
  print(f'Adding testrun to Req {response.text}')
  # TODO: Add protection against bad response
  cur.execute(f"INSERT INTO test_results (test_case_id, requirement_id, result, data_point, deviation) VALUES ({id}, '{response.text}', {req.get('passing', False)}, {req.get('value', 0)}, {req.get('deviation', 0)})")
  conn.commit()

if __name__ == '__main__':
  parser = argparse.ArgumentParser()
  parser.add_argument('-p', '--port')
  parser.add_argument('-f', '--file')
  args = parser.parse_args()
  file = args.file or file
  app.run(host='0.0.0.0', port=args.port or 8080, debug=True)


# import necessary libraries
# import os
# os.environ['SAPHIRA_URL'] = 'http://localhost:8081'
import saphira 

class Radar:
    def get_radar_reading():
        raise Exception()

# define function to test radar state machine 
test_radar_state_machine = compile(saphira.get_param('2771eea4-8deb-4e43-9462-bcbedf40a8b1.json', 'Braking'), '<string>', "exec")
        
# try to get the radar reading from Radar class 
try:
    radar_reading = Radar.get_radar_reading()
# if there is an error, update test status to False 
except Exception as e:
    saphira.update_test_status('2771eea4-8deb-4e43-9462-bcbedf40a8b1.json', 'Braking', exception=e)
# if successful, update test status to True 
else:
    saphira.update_test_status('2771eea4-8deb-4e43-9462-bcbedf40a8b1.json', 'Braking', True)


import sys

from saphira import *

project = sys.argv[1]
stdin_input = []
tests = {}
curr_test = None
for line in sys.stdin.readlines():
    if '###' in line:
        curr_test = line.strip().split('### ')[1]
        tests[curr_test] = ''
    elif curr_test:
        tests[curr_test] += line.strip() + '\n'
for test_name in tests:
    stdin_input_str = tests[test_name]
    req, s3_link = save_manual_test_input(project, test_name, stdin_input_str)
    if req != 'None':
        update_test_status(project, req, '[FAILING]' not in stdin_input_str, test=s3_link)
    else:
        print('No req matched')